# URI widget

from grokcore import component as grok
from zeam.form.ztk.fields import (
    SchemaField, SchemaFieldWidget, registerSchemaField)
from zope.schema import interfaces as schema_interfaces
from zope.interface import Interface


def register():
    registerSchemaField(URISchemaField, schema_interfaces.IURI)


class URISchemaField(SchemaField):
    """A text line field.
    """
    target = '_self'


class URIWidget(SchemaFieldWidget):
    grok.adapts(URISchemaField, Interface, Interface)


class URIDisplayWidget(SchemaFieldWidget):
    grok.adapts(URISchemaField, Interface, Interface)
    grok.name('display')

    @property
    def target(self):
        return self.component.target



# -*- coding: utf-8 -*-

from grokcore import component as grok
from zeam.form.ztk.fields import SchemaField, SchemaFieldWidget
from zope.schema import interfaces as schema_interfaces
from zeam.form.ztk.fields import registerSchemaField


def register():
    registerSchemaField(PasswordField, schema_interfaces.IPassword)


class PasswordField(SchemaField):
    """A password field.
    """


class PasswordWidget(SchemaFieldWidget):
    grok.adapts(PasswordField, None, None)



# -*- coding: utf-8 -*-

from zeam.form.ztk.fields import SchemaField
from zeam.form.ztk.fields import registerSchemaField
from zope.schema import interfaces as schema_interfaces


def register():
    registerSchemaField(FloatSchemaField, schema_interfaces.IFloat)
    registerSchemaField(IntSchemaField, schema_interfaces.IInt)


class IntSchemaField(SchemaField):
    """A integer field.
    """

class FloatSchemaField(SchemaField):
    """A float field.
    """



# -*- coding: utf-8 -*-

from grokcore import component as grok
from zeam.form.ztk.fields import SchemaField, SchemaFieldWidget
from zeam.form.ztk.fields import registerSchemaField
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces


def register():
    registerSchemaField(TextSchemaField, schema_interfaces.IText)


class TextSchemaField(SchemaField):
    """A text field.
    """


class TextFieldWidget(SchemaFieldWidget):
    grok.adapts(TextSchemaField, Interface, Interface)



# Text line widget

from grokcore import component as grok
from zeam.form.ztk.fields import (
    SchemaField, SchemaFieldWidget, registerSchemaField)
from zope.schema import interfaces as schema_interfaces


def register():
    registerSchemaField(TextLineSchemaField, schema_interfaces.ITextLine)


class TextLineSchemaField(SchemaField):
    """A text line field.
    """


class TextLineWidget(SchemaFieldWidget):
    grok.adapts(TextLineSchemaField, None, None)



# -*- coding: utf-8 -*-

from grokcore import component as grok
from zeam.form.base.markers import NO_VALUE
from zeam.form.base.widgets import WidgetExtractor, DisplayFieldWidget
from zeam.form.ztk.fields import SchemaField, SchemaFieldWidget
from zeam.form.ztk.fields import registerSchemaField
from zope.i18nmessageid import MessageFactory
from zope.schema import interfaces as schema_interfaces

_ = MessageFactory("zeam.form.base")


def register():
    registerSchemaField(BooleanSchemaField, schema_interfaces.IBool)


class BooleanSchemaField(SchemaField):
    """A boolean field.
    """


class CheckBoxWidget(SchemaFieldWidget):
    grok.adapts(BooleanSchemaField, None, None)


class CheckBoxDisplayWidget(DisplayFieldWidget):
    grok.adapts(BooleanSchemaField, None, None)

    def valueToUnicode(self, value):
        if bool(value):
            return _(u'Yes')
        return _(u'No')


class CheckBoxWidgetExtractor(WidgetExtractor):
    grok.adapts(BooleanSchemaField, None, None)

    def extract(self):
        value, error = WidgetExtractor.extract(self)
        if value is NO_VALUE:
            value = False
        elif value == 'True':
            value = True
        else:
            value = False
        return (value, error)




# -*- coding: utf-8 -*-

from zeam.form.base.datamanager import ObjectDataManager
from zeam.form.base.errors import Errors
from zeam.form.base.fields import Fields
from zeam.form.base.markers import NO_VALUE, Marker, DEFAULT
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.base.widgets import Widgets
from zeam.form.base.form import cloneFormData
from zeam.form.ztk.fields import (
    SchemaField, registerSchemaField, SchemaFieldWidget)
from zeam.form.ztk.interfaces import IObjectSchemaField

from zope.component import getUtility
from zope.component.interfaces import IFactory
from zope.interface import Interface, implements
from zope.schema import interfaces as schema_interfaces

from grokcore import component as grok


def register():
    registerSchemaField(ObjectSchemaField, schema_interfaces.IObject)


class ObjectSchemaField(SchemaField):
    """A collection field.
    """
    implements(IObjectSchemaField)
    objectFactory = DEFAULT
    dataManager = ObjectDataManager

    def __init__(self, field):
        super(ObjectSchemaField, self).__init__(field)
        self.__object_fields = Fields(field.schema)

    @property
    def objectSchema(self):
        return self._field.schema

    @property
    def objectFields(self):
        return self.__object_fields

    def getObjectFactory(self):
        if self.objectFactory is not DEFAULT:
            return self.objectFactory
        schema = self.objectSchema
        return getUtility(IFactory, name=schema.__identifier__)


class ObjectFieldWidget(SchemaFieldWidget):
    grok.adapts(ObjectSchemaField, Interface, Interface)

    def prepareContentValue(self, value):
        if value is NO_VALUE:
            return {self.identifier: []}
        return {self.identifier: value}

    def update(self):
        super(ObjectFieldWidget, self).update()
        value = self.component.dataManager(self.inputValue())
        form = cloneFormData(self.form, value, self.identifier)
        self.objectWidgets = Widgets(form=form, request=self.request)
        self.objectWidgets.extend(self.component.objectFields)
        self.objectWidgets.update()


class ObjectDisplayWidget(ObjectFieldWidget):
    grok.name('display')


class ObjectFieldExtractor(WidgetExtractor):
    grok.adapts(ObjectSchemaField, Interface, Interface)

    def extract(self):
        is_present = self.request.form.get(self.identifier, NO_VALUE)
        if is_present is NO_VALUE:
            return (NO_VALUE, None)
        value = None
        form = cloneFormData(self.form, None, self.identifier)
        data, errors = form.extractData(self.component.objectFields)
        if not errors:
            factory = self.component.getObjectFactory()
            # Create an object with values
            value = factory(**dict(filter(
                        lambda (k, v): not isinstance(v, Marker),
                        data.items())))
            return (value, None)
        return (value, Errors(*errors, identifier=self.identifier))



# -*- coding: utf-8 -*-

from grokcore import component as grok
from zeam.form.base.markers import NO_VALUE
from zeam.form.base.widgets import DisplayFieldWidget
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.ztk.fields import SchemaField, SchemaFieldWidget
from zeam.form.ztk.fields import registerSchemaField
from zope.i18n.format import DateTimeParseError
from zope.i18nmessageid import MessageFactory
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces

_ = MessageFactory("zeam-form")


def register():
    registerSchemaField(DatetimeSchemaField, schema_interfaces.IDatetime)
    registerSchemaField(DateSchemaField, schema_interfaces.IDate)


class DatetimeSchemaField(SchemaField):
    """A datetime field.
    """
    valueLength = 'short'


class DateSchemaField(SchemaField):
    """A date field.
    """
    valueLength = 'short'


class DateFieldWidget(SchemaFieldWidget):
    grok.adapts(DateSchemaField, Interface, Interface)

    valueType = 'date'

    def valueToUnicode(self, value):
        locale = self.request.locale
        formatter = locale.dates.getFormatter(
            self.valueType, self.component.valueLength)
        return formatter.format(value)


class DateWidgetExtractor(WidgetExtractor):
    grok.adapts(DateSchemaField, Interface, Interface)

    valueType = 'date'

    def extract(self):
        value, error = super(DateWidgetExtractor, self).extract()
        if value is not NO_VALUE:
            locale = self.request.locale
            formatter = locale.dates.getFormatter(
                self.valueType, self.component.valueLength)
            try:
                value = formatter.parse(value)
            except (ValueError, DateTimeParseError), error:
                return None, str(error)
        return value, error


class DatetimeFieldWidget(DateFieldWidget):
    grok.adapts(DatetimeSchemaField, Interface, Interface)

    valueType = 'dateTime'


class DatetimeWidgetExtractor(DateWidgetExtractor):
    grok.adapts(DatetimeSchemaField, Interface, Interface)

    valueType = 'dateTime'


class DateFieldDisplayWidget(DisplayFieldWidget):
    grok.adapts(DateSchemaField, Interface, Interface)

    valueType = 'date'

    def valueToUnicode(self, value):
        formatter = self.request.locale.dates.getFormatter(
            self.valueType, self.component.valueLength)
        return formatter.format(value)


class DatetimeFieldDisplayWidget(DateFieldDisplayWidget):
    grok.adapts(DatetimeSchemaField, Interface, Interface)

    valueType = 'dateTime'



# -*- coding: utf-8 -*-

from zeam.form.base.markers import NO_VALUE
from zeam.form.base.widgets import WidgetExtractor
from zeam.form.ztk.interfaces import IFormSourceBinder
from zeam.form.ztk.fields import (
    SchemaField, registerSchemaField, SchemaFieldWidget)

from zope import component
from zope.interface import Interface
from zope.schema import interfaces as schema_interfaces
from zope.schema.interfaces import IVocabularyTokenized, IVocabularyFactory
from zope.schema.interfaces import IContextSourceBinder

from grokcore import component as grok


def register():
    registerSchemaField(ChoiceSchemaField, schema_interfaces.IChoice)


class ChoiceSchemaField(SchemaField):
    """A choice field.
    """

    def __init__(self, field):
        super(ChoiceSchemaField, self).__init__(field)
        self._source = None
        self._factory = None    # The field can have a custom factory.
        self._factory_name = None
        if field.source is not None:
            if IVocabularyTokenized.providedBy(field.source):
                self._source = field.source
            else:
                self._factory = field.source
        elif isinstance(field.vocabularyName, str):
            # We delay the lookup of the vocabulary, to be sure it
            # have been registered.
            self._factory_name = field.vocabularyName

    @apply
    def factory():

        def getter(self):
            if self._factory is None:
                if self._factory_name is not None:
                    self._factory = component.getUtility(
                        schema_interfaces.IVocabularyFactory,
                        name=self._factory_name)
            return self._factory

        def setter(self, factory):
            if isinstance(factory, str):
                self._factory_name = factory
                self._factory = None
            else:
                self._factory_name = None
                self._factory = factory
            self._source = None

        return property(getter, setter)

    @apply
    def source():

        def getter(self):
            return self._source

        def setter(self, source):
            if IVocabularyTokenized.providedBy(source):
                # If that's custom, we need to re-inject this in zope.schema
                # to get validation working properly.
                self._source = source
                self._field.vocabulary = source
            else:
                # This is a factory.
                self.factory = source

        return property(getter, setter)

    def getChoices(self, form, reset=False):
        source = self.source
        if source is None or reset:
            factory = self.factory
            assert factory is not None, \
                "No vocabulary source available."
            if (IContextSourceBinder.providedBy(factory) or
                IVocabularyFactory.providedBy(factory)):
                source = factory(form.context)
            elif IFormSourceBinder.providedBy(factory):
                source = factory(form)
            assert IVocabularyTokenized.providedBy(source), \
                "No valid vocabulary available, %s is not valid for %s" % (
                source, self)
            self._field.vocabulary = source
        return source


class ChoiceFieldWidget(SchemaFieldWidget):
    grok.adapts(ChoiceSchemaField, Interface, Interface)

    def __init__(self, field, form, request):
        super(ChoiceFieldWidget, self).__init__(field, form, request)
        self.source = field
        self._choices = None

    def lookupTerm(self, value):
        choices = self.choices()
        try:
            return choices.getTerm(value)
        except LookupError:
            # the stored value is invalid. fallback on the default one.
            default = self.component.getDefaultValue(self.form)
            if default is not NO_VALUE:
                return choices.getTerm(default)
        return None

    def valueToUnicode(self, value):
        term = self.lookupTerm(value)
        if term is not None:
            return term.token
        return u''

    def choices(self):
        if self._choices is not None:
            return self._choices
        # self.source is used instead of self.component in order to be
        # able to override it in subclasses.
        self._choices = self.source.getChoices(self.form)
        return self._choices


class ChoiceDisplayWidget(ChoiceFieldWidget):
    grok.name('display')

    def valueToUnicode(self, value):
        term = self.lookupTerm(value)
        if term is not None:
            return term.title
        return u''


class ChoiceWidgetExtractor(WidgetExtractor):
    grok.adapts(ChoiceSchemaField, Interface, Interface)

    def extract(self):
        value, error = super(ChoiceWidgetExtractor, self).extract()
        if value is not NO_VALUE:
            choices = self.component.getChoices(self.form)
            try:
                value = choices.getTermByToken(value).value
            except LookupError:
                return (None, u'Invalid value')
        return (value, error)


# Radio Widget

class RadioFieldWidget(ChoiceFieldWidget):
    grok.adapts(ChoiceSchemaField, Interface, Interface)
    grok.name('radio')

    def renderableChoices(self):
        current = self.inputValue()
        base_id = self.htmlId()
        for i, choice in enumerate(self.choices()):
            yield {'token': choice.token,
                   'title': choice.title or choice.token,
                   'checked': choice.token == current and 'checked' or None,
                   'id': base_id + '-' + str(i)}
